package org.sjsustudents;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGL11;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLContext;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.egl.EGLSurface;
import javax.microedition.khronos.opengles.GL10;
import android.app.Activity;
import android.content.Context;
import android.opengl.GLU;

public class GLThread extends Thread{

	   	private final GLView view;
	   	private boolean done = false;
	   	private final GLwall wall = new GLwall();
	   	private final TopBottom  topbuttom = new TopBottom();
	   	
	   	private long startTime;
	   	private int myoption;

	   	GLThread(GLView view, int option) 
	   	{
	   		this.view = view;
	   		this.myoption = option;
	   		
	   	}
	       
	   	@Override
	   	public void run() 
	   	{
	   	   // Init OpenGL
	   	   EGL10 egl = (EGL10)EGLContext.getEGL();
	   	   EGLDisplay display = egl.eglGetDisplay(EGL10.EGL_DEFAULT_DISPLAY);
	   	   
	   	   int[] version = new int[2];
	   	   egl.eglInitialize(display, version);
	   	   
	   	   EGLConfig[] configs = new EGLConfig[1];
	   	   int[] configSpec = { EGL10.EGL_RED_SIZE, 5,
	   			   EGL10.EGL_GREEN_SIZE, 6, EGL10.EGL_BLUE_SIZE, 5,
	   			   EGL10.EGL_DEPTH_SIZE, 16, EGL10.EGL_NONE};
	   	   
	   	   int[] numConfig = new int[1];
	   	   egl.eglChooseConfig(display, configSpec, configs, 1, numConfig);
	   	   EGLConfig config = configs[0];
	   	   EGLContext glc = egl.eglCreateContext(display, 
	   			   config, EGL10.EGL_NO_CONTEXT, null);
	   	   
	   	   EGLSurface surface = egl.eglCreateWindowSurface(display, config, view.getHolder(), null);
	   	   egl.eglMakeCurrent(display, surface, surface, glc);
	   	   GL10 gl = (GL10)(glc.getGL());
	   	   init(gl);
	   	   GLwall.loadTexture(gl, view.getContext(), R.drawable.myphoto); 
	   	 
	   	   while(!done)
	   	   {
	   		   drawFrame(gl); //
	   		   egl.eglSwapBuffers(display, surface);
	   		   
	   		   //Error Handling
	   		   if(egl.eglGetError() == EGL11.EGL_CONTEXT_LOST)
	   		   {
	   			  Context c = view.getContext();
	   			  if(c instanceof Activity)
	   			  {
	   				 ((Activity) c).finish();
	   			  }
	   		   }
	   	   }
	   	   
	   	   //Free OpenGL resources
	   	   egl.eglMakeCurrent(display, EGL10.EGL_NO_SURFACE, EGL10.EGL_NO_SURFACE, 
	   			   EGL10.EGL_NO_CONTEXT);
	   	   egl.eglDestroySurface(display, surface);
	   	   egl.eglDestroyContext(display, glc);
	   	   egl.eglTerminate(display);	  	   	   
	   	
	   	}

	   	public void requestExitAndWait()
	   	{
	   		//Tell the thread to quit
	   		done = true;
	   		
	   		try
	   		{
	   			join();
	   		}
	   		catch(InterruptedException ex)
	   		{
	   		}
	   	}

	   	private void init(GL10 gl)
	   	{
	   		startTime = System.currentTimeMillis();
	   		
	   		//define the view frustrum
	   		gl.glViewport(0, 0, view.getWidth(), view.getHeight());
	   		gl.glMatrixMode(GL10.GL_PROJECTION);
	   		gl.glLoadIdentity();
	   		float ratio = (float) view.getWidth() / view.getHeight();
	   				
	   		//set viewing angle of view frustrum. ratio of its width to
	   		// height, near and far clipping planes
	   		GLU.gluPerspective(gl, 45.0f, ratio, 1, 100);		
	   		GLU.gluLookAt(gl, 0, 0, 5, 0, 0, 0, 0, 1, 0); 		//define the current look at position

	   		
	   		//pixels farther than those already drawn ignored
	   		gl.glEnable(GL10.GL_DEPTH_TEST);
	   		gl.glDepthFunc(GL10.GL_LEQUAL);
	   		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
	   				
	   		float lightAmbient[] = new float[]{ 0.2f, 0.2f, 0.2f, 1};
	   		float lightDiffuse[] = new float[]{ 1, 1, 1, 1};
	   		float[] lightPos = new float[] {1, 1, 1, 1};
	   		
	   		gl.glEnable(GL10.GL_LIGHTING);
	   		gl.glEnable(GL10.GL_LIGHT0);
	   		
	   		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_AMBIENT, lightAmbient, 0);
	   		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_DIFFUSE, lightDiffuse, 0);
	   		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_POSITION, lightPos, 0);
	   		
	   		float matAmbient[] = new float[]{1, 1, 1, 1};
	   		float matDiffuse[] = new float[]{1, 1, 1, 1};
	   		
	   		gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_AMBIENT, matAmbient, 0);
	   		gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_DIFFUSE, matDiffuse, 0);
	   		
	   		gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
	   		gl.glEnable(GL10.GL_TEXTURE_2D);	   		                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           		
	   		
	   	}
	   	
	   	private void drawFrame(GL10 gl)
	   	{
	   		//clear screen to black
	   		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
	   		
	   		//Position model so can see
	   		gl.glMatrixMode(GL10.GL_MODELVIEW);
	   		gl.glLoadIdentity();
	   		gl.glTranslatef(0, 0, 0);

	   		long elapsed = System.currentTimeMillis() - startTime;  // get the time
			
	   		// pick the corresponding option
			switch(this.myoption){
	   		    
				case 1:  
	   				gl.glRotatef(elapsed*(30f/1000f), 0, 1, 0);
	   				break;
		 
	   			case 2:
	   				gl.glRotatef(elapsed*(30f/1000f), 0, -1, 0);
		   			break;
	   		 	   
	   			case 3:
	   				gl.glRotatef(elapsed*(15f/1000f), -1, 0, 0);	
	   				break;  
	   				
	   			case 4:
	   				gl.glRotatef(elapsed*(15f/1000f), 1, 0, 0);
	   				break;
	   		
	   		    case 5:
	   				gl.glTranslatef(0, 0, (int)(elapsed*(15f/10000f)));
	   				break;  
	   				
	   			case 6:
	   				gl.glTranslatef(0, 0, (int)(-elapsed*(15f/10000f)));
	   				break;
	   				
	   			case 7:
	   				GLExpro  expro = new GLExpro(elapsed);
	   				TopBottomExpro  topBottomexpro = new TopBottomExpro(elapsed);
	   				expro.draw(gl); 
	   		        topBottomexpro.draw(gl);
	   		        break; 
	   		        
	   			default:break;

	   		    }	
			// other than break the can, we need to stop the action
		   		if(this.myoption != 7){  
		   			
		   			if(elapsed>2000){
		   				done = true;
		   			}
	   				wall.draw(gl);
	   				topbuttom.draw(gl);
	   		    }

	   	}
}
